import os
import click
import click_spinner
import podcast_summarizer.youtube as youtube
import podcast_summarizer.summarizer as summarizer
from shutil import which

@click.command()
@click.version_option()
@click.option('--summary-file', '-s', default=None, help="The file to save the summary to.")
@click.option('--transcription-file', '-t', default=None, help="The file to save the transcription to.")
@click.option('--audio-format', '-f', default=None, help="The format of the audio file. Defaults to mp3 when it cannot be infered.")
@click.option('--language', '-l', default="en", help="The language code for the language of the podcast.")
@click.argument('podcast_url_or_path')
def cli(podcast_url_or_path, summary_file, transcription_file, audio_format, language):
    """Summarizes the podcast provided by the PODCAST_URL_OR_PATH.
    
    - PODCAST_URL_OR_PATH can be a URL or a path to a local audio file.
    
    - PODCAST_URL_OR_PATH can be a YouTube URL (https://www.youtube.com/watch?v=...)

    - PODCAST_URL_OR_PATH can also be a path to a local ".txt" file containing the transcription.
    """

    if not which("ffmpeg"):
        raise Exception("ffmpeg is not installed. Please install ffmpeg.")

    if not os.getenv("OPENAI_API_KEY"):
        raise Exception("OPENAI_API_KEY is not set. Please add your API key to your environment variables.")
    
    if podcast_url_or_path.startswith("https://www.youtube.com/watch?v="):
        podcast_url_or_path = youtube.extract_from_youtube(podcast_url_or_path, transcription_file, language)
    
    if podcast_url_or_path.lower().endswith(".txt") and os.path.isfile(podcast_url_or_path):
        transcription = open(podcast_url_or_path, "r").read()
    else:
        with summarizer.openAudio(podcast_url_or_path, audio_format) as file:
            click.echo("🎙️ Transcribing...", nl=False)
            with click_spinner.spinner():
                transcription = summarizer.transcribe(file.name)
            click.echo("\r", nl=False)
            click.secho("🎙️ Transcription complete!", fg="green")

        if transcription_file:
            with open(transcription_file, "w") as transcription_file:
                transcription_file.write(transcription)
                click.secho("💾 Transcription saved to transcription.txt")
        
    click.echo("📝 Summarizing...", nl=False)
    with click_spinner.spinner():
        summary = summarizer.summarize(transcription, language)
    click.echo("\r", nl=False)
    click.secho("📝 Summary complete!", fg="green")

    if summary_file:
        with open(summary_file, "w") as summary_file:
            summary_file.write(summary)
            click.secho("💾 Summary saved to summary.txt", fg="green")

    click.echo("\n", nl=False)
    click.echo(summary)

    click.secho("\nDo you have any questions about the podcast? (Or type 'exit' to quit)", fg="blue")

    prompt = click.prompt(click.style("\n> ", fg="bright_white"), prompt_suffix="")
    while prompt.lower() != "exit":
        click.echo(summarizer.askLLM(prompt))
        prompt = click.prompt(click.style("\n> ", fg="bright_white"), prompt_suffix="")


if __name__ == '__main__':
    cli()


import os
import click
import webvtt
import yt_dlp


def extract_from_youtube(podcast_url_or_path, transcription_file=None, language="en"):
    ydl_opts = {
        'format': 'bestaudio/best',
        'skip_download': True,
        'outtmpl': 'yt-audio',
        'quiet': True,
        'writesubtitles': True,
        'writeautomaticsub': True,
        'subtitleslangs': [language],
        'postprocessors': [{
            'key': 'FFmpegExtractAudio',
            'preferredcodec': 'm4a',
        }]
    }
    with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        click.echo("📺 Looking for YouTube subtitles...")
        ydl.download(podcast_url_or_path)
        if os.path.isfile(f"yt-audio.{language}.vtt"):
            vtt = webvtt.read(f"yt-audio.{language}.vtt")
            
            if not transcription_file:
                transcription_file = "yt-audio.txt"
            if not transcription_file.endswith(".txt"):
                transcription_file += ".txt"

            with open(transcription_file, "w") as f:
                transcript = ""
                lines = []
                for line in vtt:
                    lines.extend(line.text.strip().splitlines())

                previous = None
                for line in lines:
                    if line == previous:
                        continue
                    transcript += "\n" + line
                    previous = line

                f.write(transcript)
            
            podcast_url_or_path = transcription_file
            click.echo("\r", nl=False)
            click.secho(f"💾 Saved YouTube transcription to {transcription_file}", fg="green")
            
        else:
            ydl_opts['skip_download'] = False
            with yt_dlp.YoutubeDL(ydl_opts) as ydl:
                click.echo("📺 Extracting audio from YouTube video (may take a while)...")
                ydl.download(podcast_url_or_path)
                podcast_url_or_path = "yt-audio.m4a"
                click.echo("\r", nl=False)
                click.secho("💾 Saved YouTube audio to yt-audio.m4a", fg="green")
    
    return podcast_url_or_path


import os
import openai
import urllib.request
import tempfile
import whisper


messages = []
SUPPORTED_AUDIO_FORMATS = ["m4a","mp3","webm","mp4","mpga","wav","mpeg"]

def openAudio(url_or_path, audio_format):
    if os.path.isfile(url_or_path):
        return open(url_or_path, "rb")
    try:
        if not audio_format:
            audio_format = os.path.splitext(url_or_path)[1]
            if not audio_format or audio_format not in SUPPORTED_AUDIO_FORMATS:
                audio_format = "mp3"

        audio_file = tempfile.NamedTemporaryFile(suffix=audio_format)
        with urllib.request.urlopen(url_or_path) as audio:
            audio_file.write(audio.read())
        return audio_file
    except ValueError:
        raise Exception("Invalid URL or path.")

def transcribe(mp3_path):
    model = whisper.load_model("tiny")
    result = model.transcribe(mp3_path, fp16=False)

    return result["text"]

def summarize(text, language):
    return askLLM(f'Please summarize the following podcast transcription, respond in "{language}": {text}')

def askLLM(message):
    openai.api_key = os.getenv("OPENAI_API_KEY")
    
    global messages

    messages += [{ 
        "role": "user",
        "content": message,
    }]

    completion = openai.ChatCompletion.create(
        model="gpt-3.5-turbo-16k",
        messages=messages,
        temperature=0.7,
    )

    response = completion.choices[0].message.content
    return response